#!/usr/bin/python

import os
import sys
import pygtk
pygtk.require('2.0')
import gtk

import Model
from files import collect_files
from files import clean_file_list
from files import group_files
from GUI.misc import ErrorDlg
from GUI.misc import OkCancelDlg
import config
import strings as _
from oneline import comm

class ChooserDlg(gtk.FileSelection):
    'Dialog box for selecting files.'
    mode = 'open'
    files = {} # this is a dict (key = directory).  Each dict has an
               # array of filenames (complete path)

    def __init__(self, modal=True, multiple=False):
        gtk.FileSelection.__init__(self, '/home/vasile/personal/bkrpr/free_culture')
        self.multiple = multiple # controls selection of multiple files
        if modal:
            self.set_modal(True)
        self.connect("destroy", self.file_cancel)
        self.cancel_button.connect("clicked", self.file_cancel)

    def show(self, mode):
        '''mode is 'open' or 'new', depending on whether we're expecting an
        existing book or a new one.'''
        self.mode = mode
        gtk.FileSelection.show(self)

    def ok_cb(self, path = None):
        '''Handle the file chooser ok button.

        This is where the files actually get loaded from disk, pared
        down, cleaned up and grouped.

        Can return False to keep chooser open, allowing user to select
        again.  Mostly, though, returns self.files, a dict containing
        groups of files.  Each group is a list of filespecs.  Files
        are grouped by similar filenames on the theory that files
        produced by the same camera/scan can be treated similarly for
        rotate and crop.

        We can specify a path as the param if we don't want to get it
        from the chooser.  This is useful for debugging.'''

        if not path:
            self.path = self.get_filename()
        else:
            self.path = path

        self.data_filespec = None

        comm.out("User chose %s in file chooser." % self.path)
        if os.path.isfile(self.path):
            #TODO: Allow opening of multipage doc or single-page bitmap image
            ErrorDlg("%s does not support opening individual files.  "
                      "Please choose a directory and press OK." % 
                      (config.program_name),
                      parent = self)
            return False
        elif os.path.isdir(self.path):
            ls = os.listdir(self.path)

            ## Warn if data file is or isn't there, depending on mode
            if config.data_fname in ls:
                if  self.mode == 'new':
                    result = OkCancelDlg(_.unexpected_data_file(self.path)).result
                    if (result == gtk.RESPONSE_CANCEL or 
                        result == gtk.RESPONSE_DELETE_EVENT):
                        return False
                else:
                    self.data_filespec = self.path+'/'+config.data_fname
                    comm.debug("Found data file %s" % self.data_filespec)
                    
            elif self.mode == 'open':
                    ErrorDlg(_.data_file_not_found(self.path))

            files = collect_files(self.path)

        else:
            ErrorDlg("This is not an accessible dir or file.  "
                     "Can't process it.  "
                     "Please choose a file or a directory.\n\n"
                     "Error: %s"% self.path)
            return False

        self.files = group_files(clean_file_list(files))

        if len(self.files) == 0:
            ErrorDlg("This dir does not contain any files in a "
                     "format %s can read!\n\nError dir: %s" %
                     (config.program_name, self.path))
            return False

        self.hide()
        return self.files

    def file_cancel(self, w):
        '''If the user cancels the file selection, just hide the dialog box.
        This will return control to the main window.'''
        self.hide()

