#/usr/bin/python

'''The class from which all notebook panes derive.'''

import os

import pygtk
pygtk.require('2.0')
import gtk

import config
from oneline import comm

class NotebookPane(gtk.ScrolledWindow):
    '''Parent class for panes in the notebook

    Put the pane contents in self.contents so we can destroy it as needed.'''
    status_msg = ''
    active = True
    sensitive = True

    def __init__(self):
        gtk.ScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
        self.set_shadow_type(gtk.SHADOW_NONE)
        self.connect('map', self.map_cb)
        self.connect('unmap', self.unmap_cb)
        self.content = gtk.VBox()

        # If we haven't already specified a tab name, do it now
        if not hasattr(self, 'tab'):
            self.tab = gtk.Label('Default')

    def make_tab(self, label_text='', icon_source='', buttonize = False):
        '''Pack an hbox with the icon and the label, set self.tab to that hbox.

        label_text should be a string

        icon_source - a string containing either a filespec or a stock label (i.e. gtk.STOCK_HOME)
        if icon_source is blank, we place a blank icon as a placeholder

        if icon_source isn't a stock label and icon source isn't a
        full path, we try as a relative path from config.icon_dir

        If all else fails, we go with a 16-pixel-wide blank icon.

        buttonize - Defaults to False, but if you set it to True, the
        icon is turned into a button.  Use self.tab.tab_btn to connect
        signals.
        '''

        ## Make tab
        self.tab = gtk.HBox(False, 2)
        label = gtk.Label(label_text)
        label.set_use_underline(True)
        icon = gtk.Image()

        full_path = "%s/%s" % (config.icon_dir, icon_source)

        if icon_source in gtk.stock_list_ids():
            icon.set_from_stock(icon_source, gtk.ICON_SIZE_MENU)
        elif os.path.isfile(full_path):
            if os.access(full_path, os.R_OK):
                icon.set_from_file(full_path)
            else:
                print "Can't read icon file:" % full_path
                icon_source = None
        elif os.path.isfile(icon_source):
            if os.access(icon_source, os.R_OK):
                icon.set_from_file(icon_source)
            else:
                print "Can't read icon file:" % icon_source
                icon_source = None
        else:
            icon_source = None

        if not icon_source:
            icon.set_from_file("%s/blank_icon.png" % (config.icon_dir))

        if buttonize:
            self.tab_btn = gtk.Button('')
            self.tab_btn.set_image(icon)
            self.tab.pack_start(self.tab_btn, False)
        else:
            self.tab.pack_start(icon, False)
        self.tab.pack_start(label, False)
        self.tab.show_all()

    def set_sensitive(self, sensitive, edit=True):
        '''Set/unset sensitivity.  sensitive and edit are boolean.  Set edit
        False to avoid printing the inactive message.

        This code is no longer needed, butwe might want to revive it
        in edited form later on.  I'm thinking specifically of
        disabling the Output tab until after OCR is done.  But maybe
        that's not necessary or can be done in the Go pane.'''

        if sensitive != self.sensitive:
            self.sensitive = sensitive
            if sensitive:
                gtk.HBox.set_sensitive(self, True)
            else:
                gtk.HBox.set_sensitive(self, False)
                if edit:
                    self.content.destroy()
                    self.content = gtk.VBox()
                    label = gtk.Label(
                        'This function is not active until a book is loaded.'\
                        '\n\n'\
                        'Use the File menu, above, to load a new or existing '\
                        'book ripping project.')
                    self.content.pack_start(label)
                    self.add_with_viewport(self.content)
    def set_status(self, msg):
        '''Set the status bar message.  We used to save the message for later
        use, but we don't need to do that anymore.'''
        comm.out(msg)
    def unmap_cb(self, action):
        '''Runs when a pane disappears from view.  We used to use this to
        clear the status bar, but now I'm not so sure that's the
        behavior we want.'''
        pass
    def map_cb(self, action):
        '''This placeholder method is not supposed to actually do
        anything.  Override it in specific panes.'''
        pass

    def not_implemented(self):
        'Set up a pane whose functionality is not implemented yet.'
        self.content = gtk.VBox()
        self.content.pack_start(
            gtk.Label("This function is not done yet.  "\
                          "You'll have\nto wait for future versions of Bkrpr!"))
        self.add_with_viewport(self.content)
        self.set_sensitive(False, False)
