#!/usr/bin/python

'''All the code for the Rotate pane in the main notebook goes here.'''

import pygtk
pygtk.require('2.0')
try:
    import gobject
    import gtk
except:
    print "You need to install pyGTK or GTKv2 ",
    print "or set your PYTHONPATH correctly."
    sys.exit(1)

import config
from oneline import comm
from GUI.GroupPane import GroupSubPane
from GUI.GroupPane import GroupNotebookPane
from GUI.misc import prompt_and_rbtns
from GUI.misc import set_lower
import strings as _
import Model

class RotateSubPane(GroupSubPane):
    'Rotate pane for one group'

    def __init__(self, M, id):
        '''Create the rotate options pane for a specifc group

        M is the data model.  It is an object with file, path and
        rotate attributes.  We'll create rotate groups and stick them
        in M. 

        id is the unique identifier of the group.  It's a mangled path
        of some kind.'''

        GroupSubPane.__init__(self, M, id)

        ## Ugly, but it works for now:
        ## default mode for this group is in model.RotateGroup
        ## default consistent torf for this group is in model.RotateGroup

        self.group_id = id

        ## Idiomatic switch statement
        mode = {
            Model.GUIDED: 0,
            Model.AUTO: 1,
            Model.IGNORE: 2,
            }[M.rotate.group[self.group_id].mode]

        ## Content
        self.content.pack_start(
            prompt_and_rbtns(_.rotate_mode_prompt,
                             [_.rotate_mode_guided,
                              _.rotate_mode_auto,
                              _.rotate_mode_skip],
                             self.mode_rbtn_cb, M, default=mode), False)
        self.rotated_same_prompt(M)
        self.add_with_viewport(self.content)

    def set_lower(self, widget):
        '''Set the lower part of the pane to the widget.'''
        if not hasattr(self, 'lower'):
            self.lower = gtk.VBox()        
        set_lower(self, widget, self.lower, self.content)

    def same_rbtn_cb(self, widget, num, M):
        '''Callback for when the "are they all the same rotation" radio buttons are toggled.'''
        if widget.get_active():
            M.rotate.group[self.group_id].consistent = (num == 1)
            comm.verbose("Set rotate %s consistent to %s" % (
                    self.group_id, M.rotate.group[self.group_id].consistent))

    def rotated_same_prompt(self, M):
        '''Ask user if all the images are rotated the same.'''
        ## Idiomatic switch statement
        consistent = {
            True: 1,
            False: 0,
            }[M.rotate.group[self.group_id].consistent]        
        self.set_lower(prompt_and_rbtns(
                _.rotate_consistent_prompt,
                [_.no_not_sure, _.yes], 
                self.same_rbtn_cb, M, default=consistent))

    def mode_rbtn_cb(self, widget, num, M):
        '''Callback for when the automatic/manual/skip radio buttons
        are toggled.'''
        if widget.get_active():
            if num == 0:
                M.rotate.group[self.group_id].mode = Model.GUIDED
                self.rotated_same_prompt(M)
            elif num == 1:
                M.rotate.group[self.group_id].mode = Model.AUTO
                self.set_lower(gtk.Label(''))
            else:
                M.rotate.group[self.group_id].mode = Model.IGNORE
                self.set_lower(gtk.Label(''))
            comm.verbose("Set rotate %s mode to %d" % (
                    self.group_id, M.rotate.group[self.group_id].mode))

class RotatePane(GroupNotebookPane):
    '''Main pane display rotate options

    TODO: remove inner frame from rotate notebook'''

    def __init__(self, M):
        '''Make a rotate pane for the main notebook.'''
        GroupNotebookPane.__init__(self, M, RotateSubPane)
        self.make_tab('_Rotate', '%s/stock-tool-rotate-16.png' % config.icon_dir)

    def map_cb(self, action):
        'Called when this pane is chosen and becomes visible.'
        config.help_url='help:rotate'

