#!/usr/bin/python
'The main window of the GUI.  Call this and it handles the entire GUI interface'

import pygtk
pygtk.require('2.0')
try:
    import gobject
    import gtk
except:
    print "You need to install pyGTK or GTKv2 ",
    print "or set your PYTHONPATH correctly."
    sys.exit(1)

import web
import strings as _

import config
import Model

from GUI.FileChooser import ChooserDlg
from oneline import comm
from GUI.misc import ErrorDlg
from GUI.misc import OkDlg
from GUI.preferences import Preferences
from GUI.notebook import Notebook

class Bkrpr:
    'Class that is the root of all the GUI interaction'
    ui = '''<ui>
    <menubar name="MenuBar">
      <menu action="File">
      <menuitem action="New"/>
      <menuitem action="Open"/>
        <separator />
      <menuitem action="Save"/>
        <separator />
        <menuitem action="Quit"/>
      </menu>
      <menu action="Edit">
        <menuitem action="Rotate"/>
        <menuitem action="Crop"/>
        <separator />
        <menuitem action="Preferences"/>
      </menu>
      <menu action="Help">
        <menuitem action="Help contents"/>
        <menuitem action="FAQ"/>
        <menuitem action="Licenses"/>
        <menuitem action="About Bkrpr"/>
      </menu>
    </menubar>
    </ui>'''

    path = None
    files_loaded = False

    def __init__(self):

        assert not gtk.check_version(2,6,0), gtk.check_version(2,6,0)

        # Create toplevel window
        self.window = gtk.Window()
        self.window.connect('destroy', lambda w: gtk.main_quit())
        self.window.set_size_request(600, 600)
        self.window.set_title('Bkrpr %s' % config.version)
        self.vbox = gtk.VBox()
        self.window.add(self.vbox)

        # Create a UIManager and action group
        uimanager = gtk.UIManager()
        accelgroup = uimanager.get_accel_group()
        self.window.add_accel_group(accelgroup)
        actiongroup = gtk.ActionGroup('Bkrpr')
        self.actiongroup = actiongroup

        actiongroup.add_toggle_actions(
            [('Rotate', None, '_Rotate', '<Control>r',
              'Rotate image', self.rotate_cb),
             ('Crop', None, '_Crop', '<Control>c',
              'Crop image', self.crop_cb)
             ])

        actiongroup.add_actions(
            [('Quit', gtk.STOCK_QUIT, '_Quit', None,
              'Quit the Program', self.quit_cb),
             ('File', None, '_File'),
             ('Open', gtk.STOCK_OPEN, '_Open', None,
              'Open dir or file for processing existing book', self.open_cb),
             ('New', gtk.STOCK_NEW, '_New', None,
              'New book in a dir or file for processing', self.new_cb),
             ('Save', gtk.STOCK_SAVE, '_Save', None,
              'Save current settings for later', self.save_cb),
             ('Edit', None, '_Edit'),
             ('Preferences', gtk.STOCK_PREFERENCES, '_Preferences', None,
              'Preferences', self.preferences_cb),
             ('Help', None, '_Help'),
             ('Help contents', gtk.STOCK_HELP, '_Help contents', None,
              'Help with Bkrpr', self.help_cb),
             ('FAQ', None, '_FAQ', None, 'Bkrpr FAQ', self.faq_cb),
             ('Licenses', None, '_Licenses', None, 
              'Bkrpr licensing information', self.licenses_cb),
             ('About Bkrpr', gtk.STOCK_ABOUT, '_About Bkrpr', None, 
              'About Bkrpr', self.about_cb)
             ])
        actiongroup.get_action('Save').set_sensitive(False)

        ## Add the actiongroup and XML to the uimanager
        uimanager.insert_action_group(actiongroup, 0)
        uimanager.add_ui_from_string(self.ui)

        ## Create a MenuBar, initial start text and Statusbar
        menubar = uimanager.get_widget('/MenuBar')
        self.vbox.pack_start(menubar, False)

        ## Start up text
        textview = gtk.TextView()
        textview.set_wrap_mode(gtk.WRAP_WORD)
        self.start_textbuffer = textview.get_buffer()
        self.set_start_textbuffer()
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scroll.add(textview)
        self.vbox.pack_start(self.scroll, padding=2)
        comm.set_status_bar_active(True)
        self.vbox.pack_end(comm.status_bar, False)

        ## Show main window
        self.window.show_all()

        ## Setup file chooser
        self.file_chooser = ChooserDlg('Bkrpr file selection')
        self.file_chooser.ok_button.connect('clicked', self.file_ok)

        ## Setup about dialog
        text = _.about_text
        self.about_dlg = gtk.MessageDialog(
            self.window, gtk.DIALOG_DESTROY_WITH_PARENT, 
            gtk.MESSAGE_INFO, gtk.BUTTONS_OK, text)

        ## Setup Preferences Signal
        gobject.signal_new("user_level_change", Preferences, 
                           gobject.SIGNAL_ACTION, gobject.TYPE_BOOLEAN, 
                           (gobject.TYPE_INT,))

    def set_start_textbuffer(self):
        self.start_textbuffer.set_text(_.start_text())

    ## File Dialog
    def file_ok(self, action):
        '''Handle file dialog ok button.

        When user hits Ok, we can load a new dir or open an existing
        one.'''

        f = self.file_chooser.ok_cb()
        if not self.file_chooser.data_filespec:
            if not f:
                return False
            M = Model.Model(self.file_chooser.path)
            M.pagify_files(f)
            for g in sorted(M.files):
                M.add_group(g)
        else: ## load from saved data file
            M = Model.Model.load(self.file_chooser.data_filespec)
            try:
                M = Model.Model.load(self.file_chooser.data_filespec)
                config.cont = True            
            except:
                comm.debug("Error loading %s" % self.file_chooser.data_filespec)
                ErrorDlg(_.load_error)
                return False

        try:
            self.vbox.remove(self.notebook)
        except:
            self.scroll.destroy()
        self.actiongroup.get_action('Save').set_sensitive(True)
        self.notebook = Notebook(M)
        self.vbox.pack_start(self.notebook)
        config.help_url = 'help:start'
        comm.out('Loaded %s' % M.path)
        self.M = M

    def open_cb(self, action):
        'set up the file dialog for opening a book project '
        self.file_chooser.show('open')
        comm.debug("Open done.")

    def new_cb(self, action):
        'set up the file dialog for a new book project '
        self.file_chooser.show('new')
        comm.debug("New done")

    def save_cb(self, action):
        self.M.save()
        comm.debug("Saved")

    def rotate_cb(self, action):
        'Rotate callback'
        ErrorDlg("This key sequence does not yet connect to actual functionality.  " 
                 "And, frankly, it's been so long since I stuck it in the program "
                 "that I've forgotten just what it was supposed to do.")

    def crop_cb(self, action):
        'Crop callback'
        ErrorDlg("This key sequence does not yet connect to actual functionality.  " 
                 "And, frankly, it's been so long since I stuck it in the program "
                 "that I've forgotten just what it was supposed to do.")

    def help_cb(self, action):
        web.help_contents()

    def faq_cb(self, action):
        web.faq()

    def licenses_cb(self, action):
        web.license()

    def about_cb(self, action):
        'About dialog callback'
        self.about_dlg.show()
        self.about_dlg.run()
        self.about_dlg.hide()

    def preferences_cb(self, action):
        'Launch preferences dialog.'
        p = Preferences()
        p.connect("user_level_change", self.user_level_cb)

    def user_level_cb(self, widget, int):
        '''Called when user experience level changes.  Kicks off
        whatever adjustments need to be made to the GUI when the user
        level changes.'''
        comm.debug("User level changed to %d" % config.user_level)
        self.window.emit("user_level_change", config.user_level)
        self.set_start_textbuffer()

    def quit_cb(self, act):
        'Quit callback'
        comm.out('Quitting program.  This might take a few moments...')
        gtk.main_quit()

    def main(self):
        'Main entry point for the GUI'
        gtk.main()


