#!/urs/bin/python

'''The GUI functionality below the menu bar and above the status bar
is just a notebook.  That notebook's class and its panes are here.

## Note We use button.set_image, so we need at least gtk 2.6


'''

import os
import pygtk
pygtk.require('2.0')
try:
    import gobject
    import gtk
except:
    print "You need to install pyGTK or GTKv2 ",
    print "or set your PYTHONPATH correctly."
    sys.exit(1)

import config
import strings as _
from NotebookPane import NotebookPane
from RotatePane import RotatePane
from CropPane import CropPane
from GoPane import GoPane
from oneline import comm
from GUI.misc import OkCancelDlg

class StartPane(NotebookPane):
    'Starting pane to display overall instructions'
    def __init__(self, M):
        NotebookPane.__init__(self)
        self.set_policy(gtk.POLICY_NEVER,gtk.POLICY_AUTOMATIC)
        self.make_tab('_Start', gtk.STOCK_HOME)

        self.content = gtk.VBox()
        label = gtk.Label(
            'Path: %s' % M.path +
            '\n\n')
        self.content.pack_start(label)

        self.content.pack_start(self.file_list(M))
        self.add_with_viewport(self.content)
        self.show_all()


    def map_cb(self, action):
        config.help_url = 'help:start'

    def file_list(self, M, label=None):
        '''Creates and returns an expander widget with M.files beneath it

        M is the data model
        label is the text that will mark the expander'''

        if label:
            expander = gtk.Expander(label)
        else:
            expander = gtk.Expander(label='Files used in this book')

        files_vbox = gtk.VBox()
        for d in sorted(M.files):
            hbox = gtk.HBox()
            hbox.pack_start(gtk.Label(d), False, False, 30)
            files_vbox.pack_start(hbox, False, False, 0)
            for f in M.files[d]:
                hbox = gtk.HBox()
                hbox.pack_start(gtk.Label(M.pages[f].path.replace(M.path, '')), False, False, 45)
                files_vbox.pack_start(hbox, False, False, 0)
        expander.add(files_vbox)
        return expander

class OcrPane(NotebookPane):
    'OCR options pane'
    def __init__(self):
        NotebookPane.__init__(self)
        self.make_tab('_OCR', 'stock-text-layer-16.png')
        self.not_implemented()

    def map_cb(self, action):
        'Called when this pane is chosen and becomes visible.'
        config.help_url='help:ocr'

class OutputFormatPane(NotebookPane):
    'Output formatting options pane'
    def __init__(self):
        'Pane for choosing the format of the output file (PDF, SiSU, TIF, etc.)'
        NotebookPane.__init__(self)
        self.make_tab('O_utput Format', gtk.STOCK_SAVE)
        self.not_implemented()
    def map_cb(self, action):
        'Set status when this pane comes into view.'
        if config.verbose():
            self.set_status("As soon as the basic GUI is done, I'll do this pane.")
        config.help_url='help:outputformat'

class Notebook(gtk.Notebook):
    '''Notebook containing the main panes

    TODO: Pop up a file chooser if user clicks inactive tab'''

    def __init__(self, M):
        gtk.Notebook.__init__(self)

        ## Make panes and tabs for the notebook
        self.start = StartPane(M)
        self.rotate = RotatePane(M)
        self.crop = CropPane(M)
        #self.ocr = OcrPane()
        #self.output_format = OutputFormatPane()
        self.go = GoPane(M)
        
        ## Put panes in notebook with tabs
        self.set_tab_pos(gtk.POS_LEFT)
        self.append_page(self.start, self.start.tab)
        self.append_page(self.rotate, self.rotate.tab) 
        self.append_page(self.crop, self.crop.tab)
        #self.append_page(self.ocr, self.ocr.tab)
        #self.append_page(self.output_format, self.output_format.tab)
        self.append_page(self.go, self.go.tab)

        self.go.set_modal_parent_func(self.go_modal)
        self.show_all()

    def go_modal(self, sensitive):
        '''(De)activate all the tabs except Go!'''

        self.start.set_sensitive(sensitive, False)
        self.rotate.set_sensitive(sensitive, False)
        self.crop.set_sensitive(sensitive, False)
        #self.ocr.set_sensitive(sensitive, False)
        #self.output_format.set_sensitive(sensitive, False)
