#!/usr/bin/python

'''Preferences dialog.

To keep this clean, let's not import the model.  Anything in this
dialog should be reflected in config.'''

import pygtk
pygtk.require('2.0')
try:
    import gtk
except:
    print "You need to install pyGTK or GTKv2 ",
    print "or set your PYTHONPATH correctly."
    sys.exit(1)

import webbrowser

import config
from oneline import comm

class Preferences(gtk.Window):
    '''Preferences dialog.'''

    todo = {} ## We save user choices here until they hit OK

    help_url = "http://bkrpr.org/doku.php?id=Help:Preferences"

    def __init__(self):
        gtk.Window.__init__(self)
        self.set_size_request(400, 200)
        self.set_title('Bkrpr Preferences')
        self.set_modal(True)
        padding_box = gtk.VBox()
        self.add(padding_box)

        vbox = gtk.VBox()

        label = gtk.Label("Put some helpful text here, would ya?")
        label.set_line_wrap(True)
        vbox.pack_start(label, False, padding=5)

        user_level_control = gtk.combo_box_new_text()
        user_level_control.append_text("Beginner")
        user_level_control.append_text("Intermediate")
        user_level_control.append_text("Advanced")
        user_level_control.set_active(config.user_level)
        user_level_control.connect("changed", self.user_level_cb)
        self.user_level_label = gtk.Label("How well do you know Bkrpr?")
        self.user_level_hbox = gtk.HBox()
        self.user_level_hbox.pack_start(user_level_control, False, padding=5)
        self.user_level_hbox.pack_start(self.user_level_label, False, padding=5)
        vbox.pack_start(self.user_level_hbox, False, padding=5)

        if config.intermediate():
            debug_control = gtk.combo_box_new_text()
            debug_control.append_text("Quiet")
            debug_control.append_text("Verbose")
            debug_control.append_text("Debug")
            debug_control.set_active(config.verbose_level)
            debug_control.connect("changed", self.verbose_level_cb)

            self.debug_label = gtk.Label("How chatty should Bkrpr be?")
            self.debug_hbox = gtk.HBox()
            self.debug_hbox.pack_start(debug_control, False, padding=5)
            self.debug_hbox.pack_start(self.debug_label, False, padding=5)
            vbox.pack_start(self.debug_hbox, False, padding=5)

        if config.advanced():
            self.make_connect_pack_btn('cont', vbox,
                                       'Overwrite existing intermediate and final files.')
            self.cont_btn.set_active(not config.cont)


        hbox = gtk.HBox()
        self.make_connect_pack_stock('help', hbox.pack_start)
        self.make_connect_pack_stock('ok', hbox.pack_end)
        self.make_connect_pack_stock('save', hbox.pack_end)
        self.make_connect_pack_stock('cancel', hbox.pack_end)

        vbox.pack_end(hbox, False, padding=5)
        padding_box.add(vbox)

        self.help_save =  config.help_url
        config.help_url = self.help_url
        self.show_all()

    def make_connect_pack_stock(self, name, pack):
        '''Makes, connects and pack_starts a stock button using the pack command,
        connects clicked to self.name_cb.  Returns button as self.name_btn'''
        exec('self.%s_btn = gtk.Button(stock = gtk.STOCK_%s)\n' % (name, name.upper()) + 
             'self.%s_btn.connect("clicked", self.%s_cb)\n' % (name, name) +
             'pack(self.%s_btn, False, padding=5)\n' % name +
             'btn = self.%s_btn' % name)
        return btn

    def make_connect_pack_btn(self, name, container, label):
        '''Makes, connects and pack_starts a check button into the container,
        connects clicked to self.name_cb.'''
        exec("self.%s_btn = gtk.CheckButton(label)\n" % name +
             "self.%s_btn.set_active(config.%s)\n" % (name, name) +
             "container.pack_start(self.%s_btn, False, padding=5)\n" % name +
             "self.%s_btn.connect('toggled', self.%s_cb)\n" % (name, name) +
             "btn = self.%s_btn" % name)
        return btn

    def cont_cb(self, widget):
        self.todo['cont'] = not(widget.get_active())

    def user_level_cb(self, widget):
        self.todo['user_level'] = widget.get_active()
        #TODO: We should provide more help for beginner than advanced on this option and change it right in front of the user so they can see an example of how this option works.  The only problem is that since they haven't hit OK yet, we're not actually supposed to let options take effect.  So the instant feedback might be confusing.  Maybe we should make preferences take effect immediately and not have a cancel option.  Maybe that should be the behavior only at advanced levels.

    def verbose_level_cb(self, widget):
        comm.debug("Verbose selection changed to %d, but not yet propagated to config." % widget.get_active())
        self.todo['verbose_level'] = widget.get_active()

    def save_cb(self, button):
        '''Allow user to save the current state of config for future
        sessions.  TODO: implement this'''
        comm.out("Saving of preferences not implemented yet.")
        self.update_preferences()

    def help_cb(self, button):
        '''Launch browser for context-sensitive help in the
        preferences dialog. TODO: test this func'''
        webbrowser.open(self.help_url)

    def cancel_cb(self, button):
        self.destroy()

    def destroy(self, act=None):
        config.help_url = self.help_save
        gtk.Window.destroy(self)

    def update_preferences(self):
        '''Propagate preference changes.'''
        user_level = config.user_level
        ## update config
        for field in self.todo:
            exec("config.%s = %s" % (field, self.todo[field]))

        ## comm has its own verbose and debug flags
        comm.set_verbose(config.verbose())
        comm.set_debug(config.debug())

        if user_level != config.user_level:
            self.emit("user_level_change", config.user_level)
        
    def ok_cb(self, widget):
        self.update_preferences()
        self.destroy()
