#!/usr/bin/python
import gtk
import os
import gnomecanvas
import graphic


def edge_from_degrees(degrees):
    '''degrees is a multiple of 90.  Returns EdgeSelectCanvas.TOP|BOTTOM|LEFT|RIGHT'''
    degrees = abs(degrees % 360)
    if page.rotate_degrees == 0:
        return EdgeSelectCanvas.TOP
    elif page.rotate_degrees == 180:
        return EdgeSelectCanvas.BOTTOM
    elif page.rotate_degrees == 90:
        return EdgeSelectCanvas.LEFT
    elif page.rotate_degrees == 270:
        return EdgeSelectCanvas.RIGHT

    assert False, "degrees must be a multiple of 90, not %d" % degrees


def degrees_from_edge(edge):
    '''Edge is EdgeSelectCanvas.TOP|BOTTOM|LEFT|RIGHT

    Returns corresponding degrees'''

    if edge == EdgeSelectCanvas.TOP:
        return 360
    if edge == EdgeSelectCanvas.BOTTOM:
        return 180
    if edge == EdgeSelectCanvas.LEFT:
        return 90
    if edge == EdgeSelectCanvas.RIGHT:
        return 270

class EdgeSelectCanvas(gnomecanvas.Canvas):

    TOP = 0
    BOTTOM = 1
    LEFT = 2
    RIGHT = 3
    BLANK = 4

    '''Display a graphic and let user click top, bottom, left or right edge.'''
    def __init__(self, image_spec, width=600, height=400):
	'''image_spec = image instance, graphic instance or filename of image to load
        width = max width of canvas
        height = max height of canvas'''

        ## Make graphic instance (load file if needed), then scale image
        self.im = graphic.graphic(image_spec)
        if self.im.w > width or self.im.h > height:
            self.im.thumbnail((height, width))

        ## Setup canvas
	gnomecanvas.Canvas.__init__(self, aa=True)
	self.set_size_request(self.im.w, self.im.h)
	self.set_scroll_region(0, 0, self.im.w, self.im.h)
	self.connect("button_press_event", self.btn_event)

        ## Add image to background
        self.root().add(gnomecanvas.CanvasPixbuf, pixbuf=self.im.pixbuf())

        ## Draw triangles for clicking
        self.tri = [0,1,2,3]
        for t in range(4):
            self.tri[t] = self.draw_tri(t)
                                                    
    def draw_tri(self, direction):
        '''Place a triangle on the image.  Clicking in the tri indicates an edge.'''
        if direction == self.TOP:
            return self.root().add(gnomecanvas.CanvasPolygon, fill_color_rgba=0xFF000055,
                                   points=(0,0, int(self.im.w / 2), int(self.im.h / 2),
                                           self.im.w, 0))
        if direction == self.LEFT:
            return self.root().add(gnomecanvas.CanvasPolygon, fill_color_rgba=0x00FF0055,
                                   points=(0,0, 0,self.im.h,
                                           int(self.im.w / 2), int(self.im.h / 2)))			       
        if direction == self.RIGHT:
            return self.root().add(gnomecanvas.CanvasPolygon, fill_color_rgba=0x00FF0055,
                                   points=(self.im.w,0, self.im.w,self.im.h,
                                           int(self.im.w / 2), int(self.im.h / 2)))
        if direction == self.BOTTOM:
            return self.root().add(gnomecanvas.CanvasPolygon, fill_color_rgba=0xFF000055,
                            points=(self.im.w,self.im.h, 0,self.im.h,
                                    int(self.im.w / 2), int(self.im.h / 2)))

    def activate_tri(self, direction):
        '''User has picked an edge'''

        if hasattr(self, "active"):
            if self.active != direction:
                self.tri[self.active].destroy()
                self.tri[direction] = self.draw_tri(direction)
        else:
            for a in range(4):
                if direction != a:
                    self.tri[a].destroy()

        self.active = direction

    def btn_event(self, widget, event):
        if (event.button == 1):
            m = self.im.h / (self.im.w * 1.0)

            if event.y < self.im.h - m * event.x:
                if event.y < m * event.x:
                    direc = self.TOP
                else:
                    direc = self.LEFT
            else:
                if event.y < m * event.x:
                    direc = self.RIGHT
                else:
                    direc = self.BOTTOM

            self.activate_tri(direc)

    def degrees(self):
        return degrees_from_edge(self.active)


class RotateWidget(gtk.Dialog):
    '''A frame with an EdgeSelectCanvas and some buttons.'''
    def __init__(self, image_spec, parent=None):
        gtk.Dialog.__init__(self, "Rotate Image", parent,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            ("_Blank", EdgeSelectCanvas.BLANK,
                             gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))

        self.edge_select = EdgeSelectCanvas(image_spec)
        self.vbox.pack_start(self.edge_select)
        label = gtk.Label("Click on the top edge of the page.  If the page is blank, "+
                          "choose blank.  If you make a mistake, select a "+
                          "different edge before hitting OK.  But don't worry too "+
                          "much about errors; you'll have a chance to correct them "+
                          "later.\n\n"+
                          "If hitting OK after each selection is tedious, you'll soon "+
                          "be able to set the user experience level to advanced for "+
                          "more streamlined operation.")
        label.set_line_wrap(True)
        self.vbox.pack_start(label, expand=True)

    def run(self):
        resp = gtk.Dialog.run(self)

        if resp == gtk.RESPONSE_REJECT or resp == EdgeSelectCanvas.BLANK:
            return resp

        try:
            return self.edge_select.active
        except:
            return gtk.RESPONSE_REJECT


        

if __name__ == '__main__':
    window = gtk.Window()
    window.connect('destroy', gtk.main_quit)
    #window.add(EdgeSelectCanvas("foo.png"))
    rw = RotateWidget("foo.png")
    window.show_all()
    rw.show()
    gtk.main()
