#!/usr/bin/python

'''Once I added a GUI on top of the CLI, we began to need a data model
class.  So here it is.'''

import os, sys

import config
import files
from oneline import comm

try:
    import cPickle as pickle
except:
    pass

# Page mode constants
USER = 0
AUTO = 1
DEDUCED = 2

# Page confidence constants
PROBABLE_ERROR = 0
FIT_PATTERN = 1
USER_SPECIFIED = 2

# Rotate mode constants
GUIDED = 0
AUTO = 1
IGNORE = 2

# Rotate pattern constants
ALL_SAME = 0
## AUTO = 1  this has to be the same as AUTO above
HALF_AND_HALF = 2
ALTERNATING = 3

def alphanumeric_cmp(a, b):
    '''Normally, sort treats digits as ascii tokens.  This function
    enables sort to compare series of digits as single numerical
    tokens.

    'a1b' will  come before 'a10b'
    'a2b' will also come before 'a10b'

    Numbers come before strings, so 'a1' comes before 'aa'.

    '''

    def get_token(s, count):
        ret = 0
        if s[count] >= '0' and s[count] <= '9':
            while count < len(s) and s[count] >= '0' and s[count] <= '9':
                ret = ret * 10 + int(s[count])
                count = count + 1
            return ret, count
        else:
            return s[count], count+1

    a_count = b_count = 0

    while a_count < len(a) and b_count < len(b):
        a_token, a_count = get_token(a, a_count)
        b_token, b_count = get_token(b, b_count)
        if a_token != b_token:
            return cmp(a_token, b_token)

    return cmp(len(a), len(b))

page_id_counter = 0
class Page(object):
    '''Data class for each page of the book.

    TODO: Document the fields of this object.'''


    ## These are just placeholders for now
    crop_start = -1
    crop_end = -1
    crop_method = -1
    crop_confidence = -1
    blank = False # we assume false, but might be wrong.  True is definitive.  False might be wrong

    def __init__(self, filespec):
        global page_id_counter
        self.id = page_id_counter;
        page_id_counter = page_id_counter + 1;
        self.path = filespec
        self.rotate_fname = '%s.rotate.png' % (files.basename(self.path))
        self.crop_fname = '%s.crop.png' % (files.basename(self.path))

        ##TODO: set these
        self.dir = ''
        self.original_fname = '' ## This will just be the fname, not the full path


class Model(object):
    '''The data model holds all the file info, rotate/crop options, etc.

    Hopefully this will help us make more of the program interface-agnostic.

    TODO: Document the fields we're putting in this model. 
    
    '''
    
    files = {} # this is a dict where key = a group of files.  Each
               # group entry has a list containing page ids of the
               # pages in that group.  Groups map one-to-one with
               # directories, but I don't think that's assumed and
               # might change (i.e. more sophisticated grouping might
               # be in the future)

    pages=[] # a list of all the pages in the book.  Each element in
             # the array is a page object.  These pages are not in any
             # particular order.

    def __init__(self, path):
        '''Make the data model from the path.

        Called after user loads a new directory in the
        file_chooser, but should also be used in the CLI interface at
        some point.'''

        self.rotate = Rotate()
        self.crop = Crop()
        self.files_loaded = True
        self.path = path
        self.data_filespec = self.path + '/' + config.data_fname
        self.pagify_files(files.collect_files(path))

    def load(cls, filespec):
        file = open(filespec, 'r')
        u = pickle.load(file)
        file.close( )          
        M = u['self']
        M.rotate = Rotate.unpickle(u['rotate'])
        M.crop = Crop.unpickle(u['crop'])
        M.pages=[]
        for pg in u['pages']:
            M.pages.append(pickle.loads(pg))

        return M
    load = classmethod(load)

    def save(self, filespec=None):
        '''save model M to filespec
        filespec can be None'''

        if not filespec:
            filespec = self.data_filespec

        comm.debug("Saving %s" % filespec)

        p = {}
        p['self'] = self
        p['rotate']=self.rotate.pickle()
        p['crop']=self.crop.pickle()

        ## Save pages
        p['pages'] = []
        for pg in self.pages:
            p['pages'].append(pickle.dumps(pg))

        try:
            file = open(filespec, 'w')
            pickle.dump(p, file)
            file.close( )       
        except IOError:
            comm.out("IOError: %s" % sys.exc_value)
            return False

    def remove_group(self, grp):
        'Remove a group from the file list.'
        if grp in self.files:
            del self.files[grp]

        #self.crop.group[id] = None
        #self.rotate.group[id] = None

    def pagify_files(self, f):
        '''Replaces each path in f with a page id that indexes an
        entry in self.pages[].  Creates the self.pages elements too.'''

        for k in f.keys():
            fk = sorted(f[k], alphanumeric_cmp)
            for idx in range(len(fk)):
                p = Page(fk[idx])
                p.group = k
                self.pages.append(p)
                fk[idx] = p.id
            f[k] = fk
                
        self.files = f

    def add_group(self, *arg):
        self.rotate.add_group(*arg)
        self.crop.add_group(*arg)



class RotateCrop(object):
    'Class for routines that are common to both Crop and Rotate'
    def pickle(self):
        '''Returns a picklable version of this object as a hash
        containing self, and then a field for each subobject.  The
        subobject fields contain a pickled version of the
        subobject.'''
        p = {}
        p['self'] = self
        try:
            p['lines']=self.lines
        except:
            pass
        p['group'] = {}
        for k,v in self.group.items():
            p['group'][k] = pickle.dumps(v)
        return p

    def unpickle(cls, pickled):
        '''Alternate constructor.  Assembles class from pickled data.'''
        #self = instance(cls)
        self = pickled['self']
        try:
            self.lines = pickled['lines']
        except:
            pass

        self.group = pickled['group']
        for g in self.group:
            self.group[g] = pickle.loads(self.group[g])
        return self
    unpickle = classmethod(unpickle)

class CropGroup(object):
    'Data class for cropping a group of images.'''
    mode = AUTO ## auto|ignore

    def __init__(self):
        pass

class Crop(RotateCrop):
    'Data class for cropping several groups of images.'''
    mode=AUTO  # guided|ignore
    group = {}

    lines = {} # We save info about the crop into this hash so we can
               # do statistical analysis afterwards.

    def __init__(self):
        pass
    def add_group(self, id):
        self.group[id] = CropGroup()
        self.group[id].id = id

class RotateGroup(object):
    '''data class for rotating a group of images.'''

    mode = GUIDED ## guided|auto|ignore
    consistent = False ## set to true if all images in this group are rotated the same way

    def __init__(self):
        pass

class Rotate(RotateCrop):
    '''Class for rotating a bunch of images in groups.'''
    group = {}
    def __init__(self):
        pass

    def add_group(self, id):
        self.group[id] = RotateGroup()
        self.group[id].id = id
