#!/usr/bin/python

__author__ = 'James Vasile'
__email__ = 'james@hackervisions.org'
__license__ = '''
Copyright 2006-2008 James Vasile <james@hackervisions.org>

This file is part of Bkrpr.

Bkrpr is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your
option) any later version.

Bkrpr is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''



# To make im.show work on Ubuntu: 
# 'apt-get install xloadimage; ln -s /usr/bin/xview /usr/bin/xv'

# convert file.pdf page-%03d.pgm to turn multipage pdf into a format we can use

# needs python-json package in ubuntu
# needs python-numpy package in ubuntu

import os
import sys
import getopt

import config
import cli
import GUI.bkrpr
from oneline import comm

__version__ = config.version

## TODO
# TODO: Write some routines for actually OCRing the images
# TODO: Write an interface and routines for correcting that OCR
# TODO: Documentation
# TODO: Improve accuracy of line split
# TODO: Write SiSU files after we have good OCR text
# TODO: Rotate should know which are even and which are odd pages by filename

def help_msg():
    '''print basic program description'''
    print '''Slice image of text into smaller images, each with one line. 
    '''
def usage():
    '''Print the CLI documentation string.'''
    print '''
Usage: %s [action] [options] [filespec]

ACTIONS (select one of these)
-r  --rotate\tTurn the image so the page is right-side-up
-c  --crop\tAutocrop the image to the page edge
-l  --lines\tCut into smaller images, each with one line of text
-o  --ocr\tOcr the rotated, cropped, line-split text

OPTIONS
    --90 g\t\tOverride filespec with a file glob for pages to be rotated 90 degrees clockwise
    --180 g\t\tOverride filespec with a file glob for pages to be rotated 180 degrees clockwise
    --270 g\t\tOverride filespec with a file glob for pages to be rotated 270 degrees clockwise
    --360 g\t\tOverride filespec with a file glob for pages not to be rotated (Overrides filespec)
    --brightness n\tBrightness as percentage of current brightness (default is 100)
    --continue\t\tDon't overwrite existing rotated/cropped/line-split files
    --contrast n\tContrast as percentage of current contrast (default is 100)
    --even d\t\tRotate the even-numbered pages d degrees.  d = [90|180|270|360] (uninmplemented)
    --gui\t\tUse the GUI interface
    -h, --help
    --level [on|off]\tSet to 'on' to rotate the images level after line-splitting
    --odd d\t\tRotate the odd-numbered pages d degrees.  d = [90|180|270|360] (unimplemented)
    --tilt n\t\tSpecify max tilt of text in degrees from horizon
    -v, --verbose

Filespec is a file glob, so it can be one file or a whole dir.
You might have to put the file glob in quotes.

NOTES 

--90, --180, --270, --360

Rotating is quite slow (about a minute per page) unless you can
specify which are odd and which are even pages with --90, --180, --270
and --360.  If you have pages to be rotated and cropped, but only some
need rotation, specify the ones that do not need rotation with --360.

--even, --odd

You can specify the amount to rotate even- and odd-numbered pages with
--even and --odd.  The final digit appearing in a filename determines
the even or oddness.  3rd_edition_book_7_page_34_unabridged.tif is
even, as is 3rd_edition_page_34_book_8_edited.tif


EXAMPLES

To rotate and crop a set of pages where we know which are even and
which are odd and how much to rotate each by.  Note that the even
pages don't need rotating, just cropping:

%s.py -cr --180 'odd*.tiff'  --360 'even*.tiff'

Note the need to quote the file globs.

''' % (config.bin_name, config.bin_name)

def dev():
    'Developer documentation'
    print '''DEVELOPMENT
--dev\t\tThis message
--doctest\tRun doctest on this file TODO: expand to all our modules (TODO: write doctests)
'''

def do_args(argv):
    'Check all the commandline arguments.  Pass it sys.argv[1:]'
    if len(argv) == 0:
        print "Starting GUI. Run %s --help for commandline operation." % (
            config.program_name.lower())

        config.cli = False
        return
    try:
        opts, args = getopt.getopt(argv, "cdghlrv", 
                                   ["90=", "180=", "270=", "360="
                                    "brightness=", "continue", "contrast=", 
                                    "crop", "dev", "even=", "debug", 
                                    "destination=", "doctest", "gui", "help", 
                                    "level=", "lines", "odd=", "regex=",
                                    "resume", "rotate","tilt=", "verbose"])
    except getopt.GetoptError, err:
        print str(err)
        print "--help for options"
        sys.exit(2)

    config.cli = True
    ret = {}

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            help_msg()
            sys.exit()
        elif opt in ("--90"): ret['90'] = arg
        elif opt in ("--180"): ret['180'] = arg
        elif opt in ("--270"): ret['270'] = arg
        elif opt in ("--360"): ret['360'] = arg
        elif opt in (      "--brightness"): ret['brightness'] = arg
        elif opt in ("-c", "--crop"): ret['crop'] = True        
        elif opt in (      "--continue"): config.cont = True
        elif opt in (      "--contrast"): ret['contrast'] = arg
        elif opt in ("-d", "--debug"):  config.verbose_level = config.DEBUG
        elif opt in (      "--destination"): ret['destination'] = arg
        elif opt in (      "--doctest"): 
            import doctest
            doctest.testmod()
            sys.exit()
        elif opt in (      "--dev"):
            dev()
            sys.exit()
        elif opt in (      "--even"): ret['even'] = arg
        elif opt in (      "--gui"): config.cli = False
        elif opt in ("-g"): ret['generate_html'] = True
        elif opt in (      "--hocr"): ret['hocr'] = arg
        elif opt in ("-l", "--lines"): ret['lines'] = True
        elif opt in (      "--level"): ret['level'] = arg == 'on'
        elif opt in (      "--odd"): ret['odd'] = arg
        elif opt in ("-r", "--rotate"): ret['rotate'] = True
        elif opt in (      "--regex"): ret['regex'] = arg
        elif opt in (      "--resume"): ret['resume'] = True
        elif opt in (      "--tilt"): ret['tilt'] = arg
        elif opt in ("-v", "--verbose"): config.verbose_level = config.VERBOSE
        else:
            print "Invalid commandline option: %s %s" % (opt, arg)
            print "Incidentally, if you're seeing this, "\
                "there's a bug in commandline parsing."
            usage()
            sys.exit(2)

    if not config.cli:
        return ret

    if not('90' in ret or '180' in ret or '270' in ret or '360' in ret):
        if len(args) == 0:
            print "Must specify path or image filename."
            usage()
            sys.exit(2)
        ret['path'] = args[0]

    if not('crop' in ret or 'rotate' in ret or 
           'lines' in ret or 'generate_html' in ret):
        print "Must specify at least one action: "\
            "crop (-c), rotate (-r), lines(-l)."
        usage()
        sys.exit(2)
 
    return ret
###############################################################################
def main():
    '''Main entry point for this program.  Checks command line, starts GUI
    or CLI.'''

    if os.path.exists('pause'):
        os.unlink('pause')

    opt = do_args(sys.argv[1:])

    if not config.debug:
        try:
            import psyco
            psyco.full()        
        except ImportError:
            print "psyco not found, things might be slow."

    comm.set_verbose(config.verbose())
    comm.set_debug(config.debug())

    if config.cli:
        bkrpr = cli.OcrGlob(opt)

        if 'generate_html' in opt:
            bkrpr.generate_html()

        bkrpr.main(opt)
    else:
        bkrpr = GUI.bkrpr.Bkrpr()
        bkrpr.main()

if __name__ == '__main__':
    main()

