'''OCR using Ocropus and Tesseract
Copyright 2008 James Vasile
Released under GPLv3
See LICENSE, gplv3 or http://www.gnu.org/licenses/gpl-3.0.html

Based on pytesser.py, which was released under Apache 2.0 license:
OCR in Python using the Tesseract engine from Google
http://code.google.com/p/pytesser/
by Michael J.T. O'Kelly
V 0.0.1, 3/10/07

This is a janky adaptation.  It's ugly, but it works, albeit poorly.

'''
import Image
import subprocess

import util
import errors

bin_name = 'ocroscript' # Name of executable to be called at command line
scratch_image_name = "pyocrtemp.png" # This file must be ocropus-compatible format
cleanup_scratch_flag = True  # Temporary files cleaned up after OCR operation

def do_ocr(input_filename, *arg):
	"""Calls external ocr on input file (restrictions on types),
	outputting output_filename+'txt'"""
	args = [bin_name, 'rec-tess', input_filename]
	proc = subprocess.Popen(args, stdout=subprocess.PIPE, close_fds=True)
	child_stdout = proc.stdout.read()
	retcode = proc.wait()
	if retcode!=0:
		print "OCR failed with return code %d" % retcode
		print child_stdout
		return ''

	return child_stdout
		
def image_to_string(im, cleanup = cleanup_scratch_flag):
	"""Converts im to file, applies ocr, and fetches resulting text.
	If cleanup=True, delete scratch files after operation."""
	try:
		util.image_to_scratch(im, scratch_image_name)
		text = do_ocr(scratch_image_name)
	finally:
		if cleanup:
			pass
			util.cleanup(scratch_image_name)
	return text

def image_file_to_string(filename, cleanup = cleanup_scratch_flag, graceful_errors=True):
	"""Applies ocropus to filename; or, if image is incompatible and graceful_errors=True,
	converts to compatible format and then applies ocropus.  Fetches resulting text.
	If cleanup=True, delete scratch files after operation."""
	try:
		try:
			text = do_ocr(filename)
		except errors.Tesser_General_Exception:
			if graceful_errors:
				im = Image.open(filename)
				text = image_to_string(im, cleanup)
			else:
				raise
	finally:
		if cleanup:
			util.cleanup(scratch_image_name)
	return text
	

if __name__=='__main__':
	im = Image.open('t1.png')
	text = image_to_string(im)
	print text
	try:
		text = image_file_to_string('fnord.tif', graceful_errors=False)
	except errors.Tesser_General_Exception, value:
		print "fnord.tif is incompatible filetype.  Try graceful_errors=True"
		print value
	text = image_file_to_string('fnord.tif', graceful_errors=True)
	print "fnord.tif contents:", text
	text = image_file_to_string('fonts_test.png', graceful_errors=True)
	print text


