#!/usr/bin/python

from graphic import *
import misc
import cli
import Model
from oneline import comm
import config

class rotater():
    '''Take an image (or path to an image) and return it with the text
    right-side-up.'''

    def __init__(self, image):
        self.im = misc.file_or_image(image)

    def english_letter_check(self, words):
        '''Return the number of usual letters and non-standard letters.
        
        words is an array of words.
        '''

        standard = irregular = 0
        for w in words:
            if len(w) > 0:
                for l in w:
                    if ord('A') < ord(l) < ord('z'):
                        standard  = standard + 1
                    else:
                        irregular = irregular + 1
        return standard, irregular

    def score(self):
        'Rate image as to how much of it is English-standard letters'

        text = self.im.tesseract()
        words = text.split(' ')
        return self.english_letter_check(words)

    def rotate(self, degrees=None, **opt):

        ## Handle quicky rotation
        if degrees:
            if degrees == 360:
                return graphic(self.im)
            elif degrees == 90:
                return graphic(self.im.transpose(graphic.ROTATE_270))
            elif degrees == 180:
                return graphic(self.im.transpose(graphic.ROTATE_180))
            elif degrees == 270:
                return graphic(self.im.transpose(graphic.ROTATE_90))
            elif degrees == True:
                pass
            else:
                assert False, "Can only rotate 90, 180, 270, 360, not " + degrees

            
        ## Rotation wasn't specified, so we have to figure it out the slow way

        score=[]
        score.append(self.score())
        self.im = graphic(self.im.transpose(graphic.ROTATE_90))
        score.append(self.score())
        self.im = graphic(self.im.transpose(graphic.ROTATE_90))
        score.append(self.score())
        self.im = graphic(self.im.transpose(graphic.ROTATE_90))
        score.append(self.score())

        ## find highest scoring rotation
        max = id = 0
        for c in range(4):
            right = score[c][0]
            wrong = score[c][1]
            difference = right - wrong
            if right + wrong != 0:
                percent =  float(right) / (right + wrong)
            else:
                percent = 0
            
            # we don't do anything with percent, but maybe we should
            # if we run into accuracy problems.  Also, go back to aspell.

            if percent > max:
                max = percent
                id = c

        return graphic(self.im.im.rotate(90 * (id + 1)))



def rotate(M, page_id, **opt):
    ''' Rotate fname degrees.  If degrees==False, try to figure out which
    way is up.

    Return fname of rotated file.'''

    page = M.pages[page_id]
    
    if config.cont and os.access(page.rotate_fname, os.F_OK):
        comm.debug("   Found %s.  Skipping rotate." % page.rotate_fname)
        return
    comm.verbose("   Rotating.")

    try:
        degrees = int(page.rotate_degrees) % 360
    except:
        degrees = None

    if degrees != 0:
        g = graphic(page.path)
        r = rotater(g)
        g = r.rotate(degrees)
        g.im.save(page.rotate_fname)
    else:
        page.rotate_fname = page.path

    M.save()

def rotate_ignored_group(M, g):
    '''Update the metadata for a group as if we'd rotated it 360 degrees.'''

    #if not M.files[g]:
    #    return

    for page_id in M.files[g]:
        page = M.pages[page_id]
        page.rotate_fname = page.path

def first_rotated_degrees(M, group_id):
    '''Goes through the pages in the group and returns the 
    rotation of the first rotated page, along with how many
    pages into the group that page is.'''

    for p in range(len(M.files[group_id])):
        if hasattr(M.pages[M.files[group_id][p]], "rotate_method"):
            return M.pages[M.files[group_id][p]].rotate_degrees, p
    return None


def rotate_groups(M, update_group = None, update_file = None):
    '''Do pattern or auto rotation for a group of pages.

    update_group = a func that sends text to user.  We use it to
    update on which group we're on.
        
    update_file is similar to update_group, but for files.'''

    if not update_file:
        update_file = lambda x: x

    for g in M.files:

        ## If user closed some tabs, skip them
        #if not M.files[g]:
        #    continue

        group = M.rotate.group[g]
        mode = group.mode

        if mode == Model.IGNORE:
            rotate_ignored_group(M, g)
            return

        if update_group:
            update_group("Rotating %s" % g)

        if mode == Model.AUTO or group.pattern == Model.AUTO:
            comm.out("Automatic rotation.")
            count = 1
            for page_id in M.files[g]:
                update_file(M.pages[page_id].path)
                yield count / (len(M.files[g]) * 1.0)
                rotate(M, page_id)
                count = count + 1
        elif mode == Model.GUIDED:
            degrees, idx = first_rotated_degrees(M, g)
            if group.pattern == Model.ALL_SAME:
                count = 1
                for page_id in M.files[g]:
                    update_group(M.pages[page_id].path)
                    yield count / (len(M.files[g]) * 1.0)
                    M.pages[page_id].rotate_degrees = degrees
                    rotate(M, page_id)
                    count = count + 1
            elif group.pattern == Model.HALF_AND_HALF:
                if idx > len(M.files[g]) / 2:
                    degrees = (degrees + 180) % 360
                count = 1
                for page_id in M.files[g]:
                    update_group(M.pages[page_id].path)
                    yield count / (len(M.files[g]) * 1.0)
                    M.pages[page_id].rotate_degrees = degrees
                    rotate(M, page_id)
                    if page_id == M.files[g][int(len(M.files[g]) / 2)]:
                        degrees = (degrees + 180) % 360
                    count += 1
            elif group.pattern == Model.ALTERNATING:
                if idx % 2 == 1:
                    degrees = (degrees + 180) % 360
                count = 1
                for page_id in M.files[g]:
                    update_group(M.pages[page_id].path)
                    yield count / (len(M.files[g]) * 1.0)
                    M.pages[page_id].rotate_degrees = degrees
                    rotate(M, page_id)
                    degrees = (degrees + 180) % 360
                    count = count + 1
    yield False


