#/usr/bin/python

'''Common routines that don't depend on interface.'''

from gettext import gettext as _
import config

preferences_title = _("Preferences")
about_title = _("About " + config.program_name)
license_info = _("Bkrpr is licensed under the GNU General Public License,\n"\
           "version 3 or later.  See the license text at\n"\
           "http://www.fsf.org/licensing/licenses/gpl-3.0.html\n"\
           "or in the file GPLv3, distributed with this software.\n\n")
program_description = _("Bkrpr is a set of free software and hardware that can help you "
                        "digitize books. Some of our hardware designs make it easy to "
                        "photograph all the pages in a book without harming the book. Other "
                        "designs allow you to remove the binding so the pages can be dropped "
                        "into a sheet-fed scanner. Whichever method you use, the resulting "
                        "images can be processed with our software to make user-friendly files "
                        "in a variety of formats.\n\n"
                        "From book to ebook, it is Bkrpr's goal to make every step "
                        "as simple as possible.")
copyright = _("Copyright 2008, 2009 James Vasile")
about_text = _(config.program_name + ' ' + config.version + "\n" +
              copyright + "\n\n" +
              program_description + "\n\n" +
              license_info)
load_error = _("Couldn't load data file.  It might be "
               "corrupt or from an incompatible version "
               "of " + config.program_name + 
               ".  You might have to open this "
               "directory as a new ripping project.")

def start_text():
        str = _('Welcome to Bkrpr.  This application provides tools for turning '
           'scanned images of book pages into well-formatted text or '
           'properly rotated '
           'and cropped images or PDFs.\n\n')

        if (not config.advanced()):
            str = str + _('The first thing you want to do is probably '
           'to load a new book or open '
           'an existing one.  Use the File menu, above, to do that.\n\n'

           'After you load a directory of images, a column of tabs will appear to the left.  '
           'Each represents a step in '
           'the processing of your scanned image files.  Work through the '
           'various tabs and set options as needed.  You can ignore '
           'tabs for tasks you do not plan to complete (i.e. if you are not ' 
           'going to OCR, then just ignore the OCR tab).  '
           'When you are satisfied with the options, hit GO!\n\n')
        else:
            str = str + _('You are in advanced mode.\n\n')

        if (not config.intermediate()):
            str = str + _('Bkrpr is designed to offer quite a bit of help as you work through ' \
           'this process.  When you develop more familiarity with the ' \
           'program and no longer need elaborate prompts and hints, you ' \
           'can go to the Preferences menu and raise your skill level from ' \
           'beginner to intermediate or advanced.\n\n')
        elif (config.intermediate() and not config.advanced()):
            str = str + ('You are in intermediate mode.\n\n')

        return str

def data_file_not_found(path):
    return (_("Can't find ") + config.data_fname + _(" in ") + path +
            _(".\n\nTreating as a new project."))

def unexpected_data_file(path):
    return (_("Found ") + config.data_fname + _(" in ") + path + 
            _(", which means it's not a new book.\n\n"\
                  "Press OK to ignore the data file and treat this as a new book."))

rotate_mode_prompt = _("How do you want to handle this group of images?")
rotate_mode_guided = _("Guided - faster, but requires your time and attention")
rotate_mode_auto = _("Automatic - slower, but the computer does more of the work")
rotate_mode_skip = _("Skip - this group of images does not need to be rotated")
rotate_consistent_prompt = _('Are all the images in this group rotated the same way ' + 
                             '(i.e. do all the images need the same degree of rotation '+
                             'to place the text right-side-up)?')
crop_needed_prompt = _("Does this group of images need to be cropped?")
yes = _("Yes")
yes_not_sure = _("Yes / not sure")
no = _("No")
no_not_sure = _("No / not sure")
