#!/usr/bin/python
'''
Creates a tgz archive of the latest version of the branch."

make_tgz(destination) is the sub you want.
'''

import os, sys
import re
import shutil
import subprocess

program_name = "bkrpr"
release_name = program_name  # we add version number to this further down
archive_ext = "tgz"
branch = "lp:bkrpr"
dist_area = 'bkrpr-dist-build-area'

def _rmtree(d):
    #assert d[0] == '/', "For safety sake, can only remove absolute paths, not: %s" % d
    try:
        shutil.rmtree(d)
    except:
        pass

def _mkdir(newdir):
    """works the way a good mkdir should :)
        - already exists, silently complete
        - regular file in the way, raise an exception
        - parent directory(ies) does not exist, make them as well
        From http://code.activestate.com/recipes/82465/
    """
    if os.path.isdir(newdir):
        pass
    elif os.path.isfile(newdir):
        raise OSError("a file with the same name as the desired " \
                      "dir, '%s', already exists." % newdir)
    else:
        head, tail = os.path.split(newdir)
        if head and not os.path.isdir(head):
            _mkdir(head)
        if tail:
            os.mkdir(newdir)

class ReleaseInfoError(Exception):
    pass

def make_tgz(final_destination=None):
    '''If final_destination is not None, it is is a filename where we
    will write the .tgz, and it should end in .tgz.  Otherwise, we
    will write to the default release name based on README and the
    current head revision number.

    Return the final file name written.'''
    _rmtree(dist_area) # shouldn't be needed, mostly
    ret_code = subprocess.call(['bzr', 'branch', branch, dist_area])

    ## get major and minor release numbers from README file
    readme_path = os.path.join(dist_area, "README")
    first_line = open(readme_path).readline()
    m = re.match("\s*Bkrpr\s+([0-9]+.[0-9]+)\s*", first_line)
    if not m:
      raise ReleaseInfoError("%s file missing release name on first line"
                             % readme_path)
    global release_name
    release_name += "-" + m.group(1)

    ## get revision number from trunk, use as micro release number
    last_revision_file = os.path.join(dist_area,
                                      ".bzr", "branch", "last-revision")
    assert (ret_code == 0 and os.path.exists(last_revision_file),
            "Branch didn't download.")
    r = open(last_revision_file).read()
    assert ' ' in r, "Branch seems not to have downloaded properly."
    revision = r.split(' ')[0]  # we should use partition, but server's python is old.
    release_name += "." + revision

    ## Use the full release name as the destination name.
    destination = release_name
    _rmtree(destination)
    os.rename(dist_area, destination)

    ## Archive it
    archive_name = "%s.%s" % (destination, archive_ext)
    _rmtree(os.path.join(destination, ".bzr"))
    os.system('tar -czvf %s %s' % (archive_name, destination))
    if final_destination:
      shutil.move(archive_name, final_destination)
    else:
      final_destination = archive_name
    _rmtree(destination)

    return archive_name

if __name__ == '__main__':
    print make_tgz()
